/***************************************************************************
*   Copyright (C) 2005 by Achal Dhir                                      *
*   achaldhir@gmail.com                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

// dualserver.cpp

#include <sys/types.h>
#include <limits.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/time.h>
#include <time.h>
#include <stdlib.h>
#include <limits.h>
#include <memory.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include "dualserver.h"

//Function Prototypes
char* getResult(char*);
void glocal(data2*);
WORD pQu(char*, char*);
char* fQu(char*, char*);
DWORD fLong(char*);
WORD fShort(char*);
BYTE pLong(char*, DWORD);
BYTE pShort(char*, WORD);
WORD gdmess(data1*, data2*);
DWORD sdmess(data1*, data2*);
void gvdata(data1*, data3*);
void pvdata(data1*, data3*);
void indata(data2*);
BYTE *addOp(data1*, data2*);
char* IP2String(DWORD ip);
time_t getExpiry(char *raw);
DWORD resad(data1 *req, data2 *cfig);
DWORD chad(DWORD ip, data1 *req, data2 *cfig);
DWORD alad(DWORD ip, data1 *req, data2 *cfig);
WORD gdnmess(data5*, data2*);
WORD scanloc(data5*, data2*);
void getClInfo(data5 *req);
int sdnmess(data5*, data2*);
WORD fdnmess(data5*, data2*);
WORD scancat(data5*, data2*);
WORD frdnmess(data5*, data2*);
void getdnserv(data2 *cfig);
char* cleanstr(char*, BOOL);
BOOL getSection(char *sectionName, char *buffer, int sizeoftemp, char *fileName);
BOOL writeEntry(char *sectionName, char *name, char *value, char *fileName);
int getIndex(data2 *cfig, DWORD ip);
DWORD my_inet_addr(char* str);
DWORD getNetwork(DWORD);
void addEntry(data2 *cfig, DWORD *array, DWORD entry);

//Global veriables
char tempbuff[256];
char *iniFile = "/etc/dualserver.ini";
char *leaFile = "/etc/dualserver.state";
BOOL dhcpService = true;
BOOL dnsService = true;
BOOL verbatim;
timeval tv;
fd_set readfds;
fd_set writefds;

int main(int argc, char **argv)
{
	verbatim = (argc > 1 && !strcasecmp(argv[1], "-v"));
	if (verbatim)
	{
		if (getuid())
		{
			printf("Error: Only root should run this program\n");
			exit(1);
		}
		data2 cfig;
		data1 dhcpr;
		data5 dnsr;
		indata(&cfig);

		if (dnsService && dhcpService)
			printf("Starting DNS Service..\nStarting DHCP Service..\n\nRemote DNS Servers: ");
		else if (dnsService)
			printf("Starting DNS Service..\n\nRemote DNS Servers: ");
		else
			printf("Starting DHCP Service..\n\nLocal DNS Servers: ");
		for (int i = 0; cfig.dns[i];i++)
			printf("%s ", IP2String(cfig.dns[i]));
		if (dhcpService)
		{
			printf("\nDHCP Range: %s", IP2String(htonl(cfig.rangeStart)));
			printf(" - %s", IP2String(htonl(cfig.rangeEnd)));
		}
		printf("\nDefault Gateways: ");
		for (int i = 0; cfig.routers[i];i++)
			printf("%s ", IP2String(cfig.routers[i]));

		printf("\nServer Name: %s", cfig.servername);
		printf("\nLocal Zone: %s", cfig.domainname);
		printf("\nSubnet Mast: %s", IP2String(cfig.subnetmask));
		printf("\nLocal Interfaces: ");
		if (cfig.servers[0])
			for (int i = 0; cfig.servers[i];i++)
				printf("%s ", IP2String(cfig.servers[i]));
		else
			printf("%s ", "none found");
		printf("\nLease Time: %u (sec)", cfig.lease);
		printf("\nListening On: %s", IP2String(cfig.serverip));
		printf("\n\nAccepting requests..\n");

		while (1)
		{
			tv.tv_sec = 10;
			tv.tv_usec = 0;

			FD_ZERO(&readfds);
			//FD_ZERO(&writefds);

			FD_SET(cfig.socket[0], &readfds);
			FD_SET(cfig.socket[1], &readfds);
			FD_SET(cfig.socket[2], &readfds);

			select(INT_MAX, &readfds, NULL, NULL, &tv);

			if (FD_ISSET(cfig.socket[0], &readfds))
			{
				if (gdmess(&dhcpr, &cfig))
				{
					sdmess(&dhcpr, &cfig);
					if (dhcpr.resp_type == DHCP_MESS_ACK)
						printf("Host %s alotted %s for %i seconds\n", dhcpr.hostname, IP2String(dhcpr.dhcpp.bp_yiaddr), cfig.lease);
				}
			}

			if (FD_ISSET(cfig.socket[2], &readfds))
			{
				if (frdnmess(&dnsr, &cfig))
				{
					sdnmess(&dnsr, &cfig);
					if (getResult(dnsr.raw))
						printf("%s resolved from Forwarding server as %s\n", dnsr.query, tempbuff);
				}
			}

			if (FD_ISSET(cfig.socket[1], &readfds))
			{
				if (gdnmess(&dnsr, &cfig))
				{
					if (scanloc(&dnsr, &cfig))
					{
						if (htons(dnsr.dnsp.header.ancount))
							printf("%s resolved Locally to %s\n", dnsr.query, getResult(dnsr.raw));
						sdnmess(&dnsr, &cfig);
					}
					else if (scancat(&dnsr, &cfig))
					{
						if (getResult(dnsr.raw))
							printf("%s resolved from cache as %s\n", dnsr.query, tempbuff);
						sdnmess(&dnsr, &cfig);
					}
					else if (!fdnmess(&dnsr, &cfig))
					{
						printf("Failed to forward query %s\n", dnsr.query);
						sdnmess(&dnsr, &cfig);
					}
				}
			}
		}
	}
	else
	{
		if (getuid())
		{
			syslog(LOG_MAKEPRI(LOG_LOCAL1, LOG_CRIT), "Only root should run this program");
			exit(1);
		}

		/* Our process ID and Session ID */
		pid_t pid, sid;

		/* Fork off the parent process */
		pid = fork();
		if (pid < 0)
		{
			exit(EXIT_FAILURE);
		}
		/* If we got a good PID, then
		we can exit the parent process. */
		if (pid > 0)
		{
			exit(EXIT_SUCCESS);
		}

		/* Change the file mode mask */
		umask(0);

		/* Open any logs here */

		/* Create a new SID for the child process */
		sid = setsid();
		if (sid < 0)
		{
			/* Log the failure */
			exit(EXIT_FAILURE);
		}

		/* Change the current working directory */
		if ((chdir("/")) < 0)
		{
			/* Log the failure */
			exit(EXIT_FAILURE);
		}

		/* Close out the standard file descriptors */
		close(STDIN_FILENO);
		close(STDOUT_FILENO);
		close(STDERR_FILENO);

		/* Daemon-specific initialization goes here */
		data2 cfig;
		data1 dhcpr;
		data5 dnsr;
		indata(&cfig);

		/* The Big Loop */
		while (1)
		{
			tv.tv_sec = 10;
			tv.tv_usec = 0;

			FD_ZERO(&readfds);
			//FD_ZERO(&writefds);

			FD_SET(cfig.socket[0], &readfds);
			FD_SET(cfig.socket[1], &readfds);
			FD_SET(cfig.socket[2], &readfds);

			select(INT_MAX, &readfds, NULL, NULL, &tv);

			if (FD_ISSET(cfig.socket[0], &readfds))
			{
				if (gdmess(&dhcpr, &cfig))
					sdmess(&dhcpr, &cfig);
			}

			if (FD_ISSET(cfig.socket[2], &readfds))
			{
				if (frdnmess(&dnsr, &cfig))
					sdnmess(&dnsr, &cfig);
			}

			if (FD_ISSET(cfig.socket[1], &readfds))
			{
				if (gdnmess(&dnsr, &cfig))
				{
					if (scanloc(&dnsr, &cfig))
						sdnmess(&dnsr, &cfig);
					else if (scancat(&dnsr, &cfig))
						sdnmess(&dnsr, &cfig);
					else if (!fdnmess(&dnsr, &cfig))
						sdnmess(&dnsr, &cfig);
				}
			}
		}
	}

	exit(EXIT_SUCCESS);
}

char *fQu(char *name, char *raw)
{
	char *xbuff = name;
	while (*raw)
	{
		int size = *raw;
		raw++;
		memcpy(xbuff, raw, size);
		xbuff += size;
		raw += size;
		if (!*raw)
			break;
		*xbuff = 46;
		xbuff++;
	}
	*xbuff = 0;
	return name;
}

WORD pQu(char *raw, char *name)
{
	WORD fullsize = 1;
	while (true)
	{
		char *i = strchr(name, 46);
		if (i != NULL)
		{
			int size = (DWORD)i - (DWORD)name;
			*raw = size;
			raw++;
			memcpy(raw, name, size);
			raw += size;
			name += size + 1;
			fullsize += size + 1;
		}
		else
		{
			int size = strlen(name);
			*raw = size;
			raw++;
			strcpy(raw, name);
			fullsize += size + 1;
			break;
		}
	}
	return fullsize;
}

WORD genMess(data5 *req, data2 *cfig)
{
	if (req->resp_len)
		req->dnsp.header.ancount = htons(1);
	else
	{
		req->dnsp.header.ancount = 0;
		req->dnsp.header.rcode = 3;
		req->dnsp.header.ra = 0;
	}

	if (cfig->servers[0])
	{
		req->dnsp.header.nscount = htons(1);
		req->dnsp.header.adcount = htons(1);
	}
	else
	{
		req->dnsp.header.nscount = 0;
		req->dnsp.header.adcount = 0;
	}

	req->dnsp.header.qr = 1;
	req->dnsp.header.aa = 1;

	char *raw = req->dnsp.data;

	for (int i = 1; i <= ntohs(req->dnsp.header.qdcount); i++)
	{
		if ((unsigned char)*raw > 63)
			raw += 2;
		else
			raw += strlen(raw) + 1;
		raw += 4;
	}

	//fQu(req->query,req->raw_query);

	if (req->resp_len)
	{
		//raw += pQu(raw, req->query);
		//printf("%s=\n",req->raw_query);
		strcpy(raw, req->raw_query);
		raw += strlen(req->raw_query) + 1;
		raw += pShort(raw, req->qtype);
		raw += pShort(raw, req->qclass);
		raw += pLong(raw, cfig->lease);
		if (req->qtype == DNS_TYPE_PTR)
		{
			raw += pShort(raw, req->resp_len + 2);
			raw += pQu(raw, req->response);
		}
		else
		{
			raw += pShort(raw, req->resp_len);
			memcpy(raw, req->response, req->resp_len);
			raw += req->resp_len;
		}
	}

	if (cfig->servers[0])
	{
		if (req->qtype == DNS_TYPE_PTR)
		{
			if (cfig->serverip)
				IP2String(htonl(cfig->serverip));
			else
				IP2String(htonl(cfig->servers[0]));
			strcat(tempbuff, ".in-addr.arpa");
			char *x = strchr(tempbuff, 46);
			x++;
			raw += pQu(raw, x);
		}
		else
			raw += pQu(raw, cfig->domainname);

		raw += pShort(raw, DNS_TYPE_NS);
		raw += pShort(raw, DNS_CLASS_IN);

		if (cfig->lease >= ULONG_MAX /24)
			raw += pLong(raw, ULONG_MAX);
		else
			raw += pLong(raw, 24 * cfig->lease);

		raw += pShort(raw, strlen(cfig->servername_fqn) + 2);
		raw += pQu(raw, cfig->servername_fqn);

		raw += pQu(raw, cfig->servername_fqn);
		raw += pShort(raw, DNS_TYPE_A);
		raw += pShort(raw, DNS_CLASS_IN);

		if (cfig->lease >= ULONG_MAX /24)
			raw += pLong(raw, ULONG_MAX);
		else
			raw += pLong(raw, 24 * cfig->lease);

		raw += pShort(raw, sizeof(DWORD));

		if (cfig->serverip)
			memcpy(raw, &cfig->serverip, sizeof(DWORD));
		else
			memcpy(raw, &cfig->servers[0], sizeof(DWORD));

		raw += sizeof(DWORD);
	}

	req->BytesReady = ((DWORD)raw - (DWORD)req->raw);
	return req->BytesReady;
}

WORD fShort(char *raw)
{
	return ntohs(*(WORD*)raw);
}

DWORD fLong(char *raw)
{
	return ntohl(*(DWORD*)raw);
}

BYTE pShort(char *raw, WORD data)
{
	*(WORD*)raw = htons(data);
	return sizeof(WORD);
}

BYTE pLong(char *raw, DWORD data)
{
	*(DWORD*)raw = htonl(data);
	return sizeof(DWORD);
}

WORD gdnmess(data5 *req, data2 *cfig)
{
	memset(req, 0, sizeof(data5));
	socklen_t nLen = sizeof(req->sockAddr);
	req->BytesRecd = recvfrom(cfig->socket[1],
	                          req->raw,
	                          sizeof(dns_packet),
	                          0,
	                          (LPSOCKADDR) & req->sockAddr,
	                          &nLen);
	if (req->BytesRecd < 0 || getIndex(cfig, req->sockAddr.sin_addr.s_addr) < 0)
		req->BytesRecd = 0;

	return req->BytesRecd;
}

WORD scanloc(data5 *req, data2 *cfig)
{
	if (!cfig->servers[0])
		glocal(cfig);

	if (ntohs(req->dnsp.header.qdcount) == 1 && ntohs(req->dnsp.header.opcode) == 0)
	{
		char *raw = req->dnsp.data;
		strcpy(req->raw_query, raw);
		fQu(req->query, req->raw_query);
		if (!strlen(req->query))
			return 0;
		raw += strlen(raw) + 1;
		req->qtype = fShort(raw);
		raw += 2;
		req->qclass = fShort(raw);
		if (req->qclass == DNS_CLASS_IN && req->qtype == DNS_TYPE_A)
		{
			char *dp = strchr(req->query, 46);
			if (dp != NULL)
			{
				if (strcasecmp(dp + 1, cfig->domainname))
					return 0;
				else
					*dp = 0;
			}

			if (!strcasecmp(req->query, cfig->servername))
			{
				if (cfig->serverip)
					req->responselong = cfig->serverip;
				else
					req->responselong = cfig->servers[0];
				//strcpy(req->query, cfig->servername_fqn);
				req->resp_len = 4;
			}
			else if (!strcasecmp(req->query, "localhost"))
			{
				req->responselong = my_inet_addr("127.0.0.1");
				pQu(req->raw_query, req->query);
				req->resp_len = 4;
			}
			else
			{
				int j = cfig->rangeEnd - cfig->rangeStart;
				DWORD k = time(NULL);
				for (int i = 0; i <= j; i++)
				{
					if (!strcasecmp(req->query, cfig->clients[i].hostname) && cfig->clients[i].expiry > k)
					{
						req->responselong = htonl(cfig->rangeStart + i);
						//sprintf(req->query, "%s.%s", cfig->clients[i].hostname, cfig->domainname);
						req->resp_len = 4;
						break;
					}
				}
			}
			return genMess(req, cfig);
		}
		else if (req->qclass == DNS_CLASS_IN && req->qtype == DNS_TYPE_PTR)
		{
			//strlwr(req->query);
			//strcpy(tempbuff, req->query);
			char *dp = strstr(req->query, ".in-addr.arpa");
			if (dp == NULL)
				dp = strstr(req->query, ".IN-ADDR.ARPA");
			if (dp != NULL)
				*dp = 0;
			DWORD arpa_addr = htonl(my_inet_addr(req->query));
			DWORD k = time(NULL);
			if (arpa_addr == cfig->servers[0] || arpa_addr == cfig->servers[1] || arpa_addr == cfig->servers[2] || arpa_addr == cfig->servers[3] || arpa_addr == cfig->servers[4])
			{
				strcpy(req->response, cfig->servername_fqn);
			}
			else if (arpa_addr == my_inet_addr("127.0.0.1"))
			{
				strcpy(req->response, "localhost");
			}
			else if (getIndex(cfig, arpa_addr) < 0)
				return 0;
			else
			{
				int i = getIndex(cfig, arpa_addr);
				if (strlen(cfig->clients[i].hostname) && cfig->clients[i].expiry > k)
				{
					sprintf(req->response, "%s.%s", cfig->clients[i].hostname, cfig->domainname);
				}
			}
			req->resp_len = strlen(req->response);
			return genMess(req, cfig);
		}
	}
	return 0;
}

WORD scancat(data5 *req, data2 *cfig)
{
	if (!strlen(req->query))
		return 0;
	time_t ctime = time(NULL);
	WORD xid = req->dnsp.header.xid;
	for (int i = 0; i < DNS_CACHE_MAX; i++)
	{
		if (!strcasecmp(req->query, cfig->cache[i].hostname))
			if (req->qtype == cfig->cache[i].qtype && cfig->cache[i].expiry >= ctime)
			{
				memcpy(req->raw, &cfig->cache[i].dnsp, cfig->cache[i].bytes);
				req->BytesReady = cfig->cache[i].bytes;
				req->dnsp.header.xid = xid;
				return req->BytesReady;
			}
	}
	return 0;
}

int sdnmess(data5 *req, data2 *cfig)
{
	req->BytesSent = sendto(cfig->socket[1],
	                        req->raw,
	                        req->BytesReady,
	                        0,
	                        (LPSOCKADDR) & req->sockAddr,
	                        sizeof(req->sockAddr));
	return req->BytesSent;
}


WORD fdnmess(data5 *req, data2 *cfig)
{
	if (!req->dnsp.header.rd)
		return 0;

	if (!cfig->dns[0])
		getdnserv(cfig);

	int nRet = -1;
	int i;
	for (i = 0; i < 5; i++)
	{
		if (!cfig->dns[0])
			break;
		cfig->sockAddr[2].sin_family = AF_INET;
		cfig->sockAddr[2].sin_addr.s_addr = cfig->dns[0];
		cfig->sockAddr[2].sin_port = htons(IPPORT_DNS);

		nRet = sendto(cfig->socket[2],
		              req->raw,
		              req->BytesRecd,
		              0,
		              (LPSOCKADDR) & cfig->sockAddr[2],
		              sizeof(cfig->sockAddr[2]));

		if (nRet > 0)
			break;
		else
		{
			for (int j = 1; j < 5; j++)
				cfig->dns[j - 1] = cfig->dns[j];
			cfig->dns[4] = 0;
		}
	}

	if (nRet > 0)
	{
		BOOL found = false;
		for (int i = cfig->queries_awaited; i && !found; i--)
			if (req->dnsp.header.xid == cfig->queue[i - 1].xid)
				found = true;

		if (!found)
		{
			if (cfig->queries_awaited == DNS_FORWARDS_MAX)
			{
				int i;
				for (i = 0; i < DNS_FORWARDS_MAX - 1; i++)
					memcpy(&cfig->queue[i], &cfig->queue[i + 1], sizeof(data6));
				cfig->queries_awaited--;
			}
			cfig->queue[cfig->queries_awaited].xid = req->dnsp.header.xid;
			cfig->queue[cfig->queries_awaited].qtype = req->qtype;
			memcpy(&cfig->queue[cfig->queries_awaited].source, &req->sockAddr, sizeof(SOCKADDR_IN));
			strcpy(cfig->queue[cfig->queries_awaited].hostname, req->query);
			cfig->queries_awaited++;
		}
		return (nRet);
	}
	else
	{
		req->dnsp.header.ancount = 0;
		req->dnsp.header.rcode = 3;
		req->dnsp.header.ra = 0;
		req->dnsp.header.qr = 1;
		req->dnsp.header.aa = 0;
		req->BytesReady = req->BytesRecd;
		return 0;
	}
}

WORD frdnmess(data5 *req, data2 *cfig)
{
	socklen_t nRet = sizeof(cfig->sockAddr[2]);
	req->BytesReady = recvfrom(cfig->socket[2],
	                           req->raw,
	                           sizeof(dns_packet),
	                           0,
	                           (LPSOCKADDR) & cfig->sockAddr[2],
	                           &nRet);

	if (req->BytesReady <= 0)
		return 0;

	if (!cfig->queries_awaited)
		return 0;

	BOOL found = false;
	for (int i = 0; i < cfig->queries_awaited; i++)
		if (!found)
		{
			if (cfig->queue[i].xid == req->dnsp.header.xid)
			{
				found = true;
				memcpy(&req->sockAddr, &cfig->queue[i].source, sizeof(SOCKADDR_IN));
				cfig->queue[i].xid = 0;
				strcpy(req->query, cfig->queue[i].hostname);
				cfig->queries_awaited--;

				if (strlen(req->query) && ntohs(req->dnsp.header.ancount) >= 1)
				{
					time_t expiry = getExpiry(req->raw);
					time_t earliest = expiry;
					int j;
					for (j = 0; j < DNS_CACHE_MAX; j++)
						if (!strcasecmp(cfig->cache[j].hostname, cfig->queue[i].hostname) && cfig->cache[j].qtype == cfig->queue[i].qtype)
							break;

					if (j == DNS_CACHE_MAX)
					{
						for (int k = DNS_CACHE_MAX - 1; k >= 0; k--)
						{
							if (cfig->cache[k].expiry < earliest)
							{
								j = k;
								earliest = cfig->cache[k].expiry;
							}
						}
					}

					if (j < DNS_CACHE_MAX)
					{
						cfig->cache[j].expiry = expiry;
						strcpy(cfig->cache[j].hostname, cfig->queue[i].hostname);
						cfig->cache[j].qtype = cfig->queue[i].qtype;
						cfig->cache[j].bytes = req->BytesReady;
						memcpy(&cfig->cache[j].dnsp, req->raw, sizeof(dns_packet));
					}
				}
			}
		}
		else
			memcpy(&cfig->queue[i - 1], &cfig->queue[i], sizeof(data6));

	if (!found)
		req->BytesReady = 0;

	req->dnsp.header.aa = 0;
	return req->BytesReady;
}

char* getResult(char *raw)
{
	dns_header *dnsheader = (dns_header *) raw;
	WORD i;

	raw += sizeof(struct dns_header);

	for (i = 1; i <= ntohs(dnsheader->qdcount); i++)
	{
		if ((unsigned char)*raw > 63)
			raw += 2;
		else
			raw += strlen(raw) + 1;
		raw += 4;
	}

	for (i = 1; i <= ntohs(dnsheader->ancount); i++)
	{
		if ((unsigned char)*raw > 63)
			raw += 2;
		else
			raw += strlen(raw) + 1;
		int type = fShort(raw);
		raw += 2; //type
		raw += 2; //class
		raw += 4; //ttl
		int dlen = fShort(raw);
		raw += 2; //datalength
		if (type == DNS_TYPE_A)
			return IP2String(*(DWORD*)raw);
		else if (type == DNS_TYPE_PTR)
			return fQu(tempbuff, raw);
		else
			raw += dlen;
	}
	return NULL;
}

time_t getExpiry(char *raw)
{
	time_t ctime = time(NULL);

	dns_header *dnsheader = (dns_header *) raw;

	raw += sizeof(struct dns_header);

	for (int i = 1; i <= ntohs(dnsheader->qdcount); i++)
	{
		if ((unsigned char)*raw > 63)
			raw += 2;
		else
			raw += strlen(raw) + 1;
		raw += 4;
	}

	for (int i = 1; i <= ntohs(dnsheader->ancount); i++)
	{
		if ((unsigned char)*raw > 63)
			raw += 2;
		else
			raw += strlen(raw) + 1;
		raw += 4;
		DWORD expiry = fLong(raw);
		if (ULONG_MAX - ctime < expiry)
			return LONG_MAX;
		else
			return (expiry + ctime);
	}
	return 0;
}

BYTE *gvdata(BYTE *raw, data1 *req, data3 *op)
{
	while (*raw == 0)
		if (raw == req->dhcpp.bp_vend + DHCP_VENDORDATA_SIZE)
			return NULL;
		else
			raw++;

	if (*raw == DHCP_OPTION_END)
		return NULL;

	op->opt_code = *raw;
	raw++;
	op->size = *raw;
	raw++;
	memcpy(op->value, raw, op->size);
	raw += op->size;
	//printf("\n%u %u",op->opt_code, op->size);
	return raw;
}

BYTE *pvdata(BYTE *raw, data3 *op)
{
	*raw = op->opt_code;
	raw++;
	*raw = op->size;
	raw++;
	memcpy(raw, op->value, op->size);
	raw += op->size;
	return raw;
}

WORD gdmess(data1 *req, data2 *cfig)
{
	//printf("\nHere");
	//int k;
	//getsockname(cfig->socket[0], (SOCKADDR*)&req->inSockAddr, &k);
	//printf("\nLocal1=%s", inet_ntoa(req->inSockAddr.sin_addr));

	memset(req, 0, sizeof(data1));

	socklen_t nLen = sizeof(req->sockAddr);
	req->BytesRecd = recvfrom(cfig->socket[0],
	                          req->raw,
	                          sizeof(dhcp_packet),
	                          0,
	                          (LPSOCKADDR) & req->sockAddr,
	                          &nLen);

	if (req->BytesRecd <= 0)
		return 0;

	if (!cfig->servers[0])
		glocal(cfig);

	if (!cfig->servers[0])
		return 0;

	data3 op;
	BYTE *raw = req->dhcpp.bp_vend;

	while (raw = gvdata(raw, req, &op))
	{
		switch (op.opt_code)
		{
		case DHCP_OPTION_MESSAGETYPE:
			req->req_type = op.value[0];
			break;

		case DHCP_OPTION_SERVERID:
			req->server = op.longvalue;
			break;

		case DHCP_OPTION_REQUESTEDIPADDR:
			req->reqIP = op.longvalue;
			break;

		case DHCP_OPTION_HOSTNAME:
			if (op.size)
			{
				memcpy(req->hostname, op.value, op.size);
				req->hostname[op.size] = 0;
				char *dp = strchr(req->hostname, 46);
				if (dp != NULL)
					*dp = 0;
			}
			break;
		}
	}

	return req->req_type;
}

DWORD resad(data1 *req, data2 *cfig)
{
	int proposed_candidate = -1;
	int expired_candidate = -1;
	int i;
	BYTE empty[8];
	memset(empty, 0, sizeof(empty));
	time_t currentTime = time(NULL);

	if (!cfig->servers[0])
		return 0;

	int j = cfig->rangeEnd - cfig->rangeStart;

	for (i = 0; i <= j; i++)
	{
		if (!memcmp(req->dhcpp.bp_chaddr, cfig->clients[i].chaddr, sizeof(empty)))
			break;
		else if (!cfig->clients[i].reserved && proposed_candidate < 0 && !memcmp(cfig->clients[i].chaddr, empty, sizeof(empty)))
			proposed_candidate = i;
		else if (!cfig->clients[i].reserved && expired_candidate < 0 && memcmp(cfig->clients[i].chaddr, empty, sizeof(empty)) && cfig->clients[i].expiry < currentTime)
			expired_candidate = i;
	}

	if (i <= j)
		proposed_candidate = i;
	else if (proposed_candidate < 0 && expired_candidate >= 0)
		proposed_candidate = expired_candidate;
	else if (proposed_candidate < 0)
		return 0;

	if (strlen(req->hostname))
		strcpy(cfig->clients[proposed_candidate].hostname, req->hostname);
	else if (strlen(cfig->clients[proposed_candidate].hostname))
		strcpy(req->hostname, cfig->clients[proposed_candidate].hostname);

	memcpy(cfig->clients[proposed_candidate].chaddr, req->dhcpp.bp_chaddr, 8);
	cfig->clients[proposed_candidate].expiry = currentTime + 120;
	return htonl(cfig->rangeStart + proposed_candidate);
}

DWORD chad(DWORD ip, data1 *req, data2 *cfig)
{
	int j = getIndex(cfig, ip);

	if (j >= 0)
		if (!memcmp(req->dhcpp.bp_chaddr, cfig->clients[j].chaddr, 8))
		{
			if (strlen(req->hostname))
				strcpy(cfig->clients[j].hostname, req->hostname);
			else if (strlen(cfig->clients[j].hostname))
				strcpy(req->hostname, cfig->clients[j].hostname);

			return ip;
		}
	return 0;
}

DWORD alad(DWORD ip, data1 *req, data2 *cfig)
{
	int j = getIndex(cfig, ip);

	if (j >= 0)
	{
		cfig->clients[j].expiry = time(NULL) + cfig->lease;

		if (strlen(req->hostname))
		{
			for (int i = 0; i <= cfig->rangeEnd - cfig->rangeStart; i++)
				if (!strcasecmp(req->hostname, cfig->clients[i].hostname))
					cfig->clients[i].hostname[0] = 0;
			strcpy(cfig->clients[j].hostname, req->hostname);
		}

		FILE *f = fopen(leaFile, "r+");
		if (f)
		{
			fseek(f, 4 + j*sizeof(data8), SEEK_SET);
			fwrite(&cfig->clients[j], sizeof(data8), 1, f);
			fclose(f);
		}
		return ip;
	}
	return 0;
}

DWORD sdmess(data1 *req, data2 *cfig)
{
	if (req->req_type == DHCP_MESS_DECLINE)
	{
		req->dhcpp.bp_yiaddr = resad(req, cfig);
		int j = getIndex(cfig, req->dhcpp.bp_yiaddr);
		if (j >= 0)
			cfig->clients[j].reserved = 1;
		return 0;
	}
	else if (req->req_type == DHCP_MESS_INFORM)
		return 0;
	else if (req->req_type == DHCP_MESS_DISCOVER && strcasecmp(req->hostname, cfig->servername))
	{
		req->resp_type = DHCP_MESS_OFFER;
		req->dhcpp.bp_yiaddr = resad(req, cfig);
		if (!req->dhcpp.bp_yiaddr)
			return 0;
	}
	else if (req->req_type == DHCP_MESS_REQUEST)
	{
		if (req->server)
			if (req->server == cfig->servers[0])
				if (req->reqIP && chad(req->reqIP, req, cfig))
				{
					req->resp_type = DHCP_MESS_ACK;
					req->dhcpp.bp_yiaddr = req->reqIP;
					//req->dhcpp.bp_yiaddr = alad(req->reqIP, req, cfig);
				}
				else if (req->dhcpp.bp_ciaddr && chad(req->dhcpp.bp_ciaddr, req, cfig))
				{
					req->resp_type = DHCP_MESS_ACK;
					req->dhcpp.bp_yiaddr = req->dhcpp.bp_ciaddr;
					//req->dhcpp.bp_yiaddr = alad(req->dhcpp.bp_ciaddr, req, cfig);
				}
				else
				{
					req->resp_type = DHCP_MESS_DECLINE;
					req->dhcpp.bp_yiaddr = 0;
				}
			else
				return 0;
		else if (req->dhcpp.bp_ciaddr)
			if (req->dhcpp.bp_ciaddr && chad(req->dhcpp.bp_ciaddr, req, cfig))
			{
				req->resp_type = DHCP_MESS_ACK;
				req->dhcpp.bp_yiaddr = req->dhcpp.bp_ciaddr;
				//req->dhcpp.bp_yiaddr = alad(req->dhcpp.bp_ciaddr, req, cfig);
			}
			else
			{
				req->resp_type = DHCP_MESS_DECLINE;
				req->dhcpp.bp_yiaddr = 0;
			}
		else if (req->reqIP)
		{
			if (req->reqIP && chad(req->reqIP, req, cfig))
			{
				req->resp_type = DHCP_MESS_ACK;
				req->dhcpp.bp_yiaddr = req->reqIP;
				//req->dhcpp.bp_yiaddr = alad(req->reqIP, req, cfig);
			}
			else
			{
				req->resp_type = DHCP_MESS_DECLINE;
				req->dhcpp.bp_yiaddr = 0;
			}
		}
		else
			return 0;
	}
	else
		return 0;

	int j = (DWORD)addOp(req, cfig) - (DWORD) & req->dhcpp;

	req->sockAddr.sin_family = AF_INET;
	req->sockAddr.sin_port = htons(IPPORT_DHCPC);

	if (req->dhcpp.bp_ciaddr)
		req->sockAddr.sin_addr.s_addr = req->dhcpp.bp_ciaddr;
	else
		req->sockAddr.sin_addr.s_addr = INADDR_BROADCAST;

	req->BytesSent = sendto(cfig->socket[0],
	                        req->raw,
	                        j,
	                        0,
	                        (sockaddr*) & req->sockAddr,
	                        sizeof(req->sockAddr));

	if (req->resp_type == DHCP_MESS_ACK && req->BytesSent > 0 && req->dhcpp.bp_yiaddr)
		return alad(req->dhcpp.bp_yiaddr, req, cfig);

	return 0;
}

BYTE *addOp(data1 *req, data2 *cfig)
{
	data3 op;
	int i;

	memset(req->dhcpp.bp_vend, 0, DHCP_VENDORDATA_SIZE);
	BYTE *raw = req->dhcpp.bp_vend;

	op.opt_code = DHCP_OPTION_MESSAGETYPE;
	op.size = 1;
	op.value[0] = req->resp_type;
	raw = pvdata(raw, &op);

	op.opt_code = DHCP_OPTION_NETMASK;
	op.size = 4;
	op.longvalue = cfig->subnetmask;
	raw = pvdata(raw, &op);

	op.opt_code = DHCP_OPTION_SERVERID;
	op.size = 4;
	if (cfig->serverip)
		op.longvalue = cfig->serverip;
	else
		op.longvalue = cfig->servers[0];
	raw = pvdata(raw, &op);

	if (req->resp_type != DHCP_MESS_DECLINE)
	{
		op.opt_code = DHCP_OPTION_NETBIOSNODETYPE;
		op.size = 1;
		op.value[0] = cfig->nodetype;
		raw = pvdata(raw, &op);

		op.opt_code = DHCP_OPTION_DOMAINNAME;
		op.size = strlen(cfig->domainname);
		memcpy(op.value, cfig->domainname, op.size);
		raw = pvdata(raw, &op);

		if (strlen(req->hostname))
		{
			op.opt_code = DHCP_OPTION_HOSTNAME;
			op.size = strlen(req->hostname);
			memcpy(op.value, req->hostname, op.size);
			raw = pvdata(raw, &op);
		}

		for (i = 0; i < 5; i++)
			if (!cfig->routers[i])
				break;
		op.opt_code = DHCP_OPTION_ROUTER;
		op.size = i * 4;
		memcpy(op.value, cfig->routers, op.size);
		raw = pvdata(raw, &op);

		if (dnsService)
		{
			op.opt_code = DHCP_OPTION_DNS;
			if (cfig->serverip)
			{
				op.longvalue = cfig->serverip;
				op.size = 4;
			}
			else
			{
				for (i = 0; i < 5; i++)
					if (!cfig->servers[i])
						break;
				op.size = i * 4;
				memcpy(op.value, cfig->servers, op.size);
			}
			raw = pvdata(raw, &op);
		}
		else
		{
			for (i = 0; i < 5; i++)
				if (!cfig->dns[i])
					break;
			op.opt_code = DHCP_OPTION_DNS;
			op.size = i * 4;
			memcpy(op.value, cfig->dns, op.size);
			raw = pvdata(raw, &op);
		}

		op.opt_code = DHCP_OPTION_IPADDRLEASE;
		op.size = 4;
		op.longvalue = htonl(cfig->lease);
		raw = pvdata(raw, &op);

		strcpy(req->dhcpp.bp_sname, cfig->servername_fqn);
	}

	*raw = DHCP_OPTION_END;
	req->dhcpp.bp_op = DHCP_REPLY;

	return raw;
}

void indata(data2 *cfig)
{
	memset(cfig, 0, sizeof(data2));

	cfig->rangeStart = htonl(my_inet_addr("192.168.0.1"));
	cfig->rangeEnd = cfig->rangeStart + DHCP_MAX_CLIENTS - 1;
	cfig->lease = 36000;
	cfig->nodetype = 8;

	char iniStr[8192];

	if (getSection("IP-RANGE", iniStr, sizeof(iniStr), iniFile))
	{
		char *iniStrPtr = cleanstr(iniStr, false);
		if (strlen(iniStrPtr))
		{
			DWORD rs = htonl(my_inet_addr(iniStrPtr));
			DWORD re = htonl(my_inet_addr(cleanstr(iniStrPtr, true)));

			if (rs && rs != INADDR_NONE && re && re != INADDR_NONE && (rs % 256) != 0 && (re % 256 != 255) && (rs / 256) == (re / 256) && rs <= re)
			{
				cfig->rangeStart = rs;
				cfig->rangeEnd = re;
			}
			else if (verbatim)
				printf("Warning:Invalid DHCP range in ini file, ignored\n");
			else
				syslog(LOG_MAKEPRI(LOG_LOCAL1, LOG_NOTICE), "Warning:Invalid DHCP range in ini file, ignored");
		}
	}

	cfig->network = getNetwork(htonl(cfig->rangeStart));

	if (getSection("SERVICES", iniStr, sizeof(iniStr), iniFile))
	{
		dhcpService = false;
		dnsService = false;
		char *iniStrPtr = cleanstr(iniStr, false);
		for (int i = 0; i < 2 && strlen(iniStrPtr); i++, iniStrPtr = cleanstr(iniStrPtr, true))
			if (!strcasecmp(iniStrPtr, "DNS"))
				dnsService = true;
			else if (!strcasecmp(iniStrPtr, "DHCP"))
				dhcpService = true;
		if (!dhcpService && !dnsService)
		{
			dhcpService = true;
			dnsService = true;
		}
	}

	if (dhcpService)
	{
		FILE *f = fopen(leaFile, "r");
		if (f)
		{
			DWORD rangeStart = 0;
			fread(&rangeStart, 4, 1, f);
			if (rangeStart == cfig->rangeStart)
				fread(cfig->clients, sizeof(data8), DHCP_MAX_CLIENTS, f);
			fclose(f);
		}

		f = fopen(leaFile, "w");
		if (f)
		{
			fwrite(&cfig->rangeStart, 4, 1, f);
			fwrite(cfig->clients, sizeof(data8), DHCP_MAX_CLIENTS, f);
			fclose(f);
		}

		if (getSection("STATIC-IPS", iniStr, sizeof(iniStr), iniFile))
		{
			char *iniStrPtr = cleanstr(iniStr, false);
			while (strlen(iniStrPtr))
			{
				char name[256];
				strcpy(name, iniStrPtr);
				char *value = strchr(name, 61);
				if (value != NULL)
				{
					*value = 0;
					value++;
					DWORD inetAddr = my_inet_addr(value);
					int i = getIndex(cfig, inetAddr);
					if (i >= 0)
					{
						//printf("\n%s=%u",value,i);
						WORD tname[8];
						sscanf(name, "%02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X", &tname[0], &tname[1], &tname[2], &tname[3], &tname[4], &tname[5], &tname[6], &tname[7]);
						for (int j = 0; j < 8; j++)
							cfig->clients[i].chaddr[j] = tname[j];

						BYTE empty[8];
						memset(empty, 0, sizeof(empty));
						for (int j = 0; j <= (cfig->rangeEnd - cfig->rangeStart); j++)
						{
							if (i != j && !memcmp(cfig->clients[i].chaddr, cfig->clients[j].chaddr, sizeof(empty)))
								memset(&cfig->clients[j], 0, sizeof(data8));
						}
						cfig->clients[i].reserved = 1;
						//cfig->clients[i].expiry = ULONG_MAX;
					}
				}
				iniStrPtr = cleanstr(iniStrPtr, true);
			}
		}
	}

	if (getSection("DOMAIN-NAME", iniStr, sizeof(iniStr), iniFile))
		strcpy(cfig->domainname, cleanstr(iniStr, false));

	if (getSection("LISTEN-ON", iniStr, sizeof(iniStr), iniFile))
		cfig->serverip = my_inet_addr(cleanstr(iniStr, false));

	if (getSection("ROUTERS", iniStr, sizeof(iniStr), iniFile))
	{
		char *iniStrPtr = cleanstr(iniStr, false);
		for (int i = 0; i < 5, strlen(iniStrPtr); i++, iniStrPtr = cleanstr(iniStrPtr, true))
			addEntry(cfig, cfig->routers, my_inet_addr(iniStrPtr));
	}
	cfig->routerSpecified = (cfig->routers[0] > 0);

	if (getSection("LEASE-SECONDS", iniStr, sizeof(iniStr), iniFile))
	{
		char *iniStrPtr = cleanstr(iniStr, false);
		char temp[100];
		sprintf(temp,"%u",ULONG_MAX);
		if ((strlen(temp) > strlen(iniStrPtr) || (strlen(temp) == strlen(iniStrPtr) && strcmp(temp,iniStrPtr) >= 0)) && atoi(iniStrPtr))
			cfig->lease = atoi(iniStrPtr);
	}

	if (getSection("NETBIOS-NODE-TYPE", iniStr, sizeof(iniStr), iniFile))
	{
		char *iniStrPtr = cleanstr(iniStr, false);
		if (atoi(iniStrPtr))
			cfig->nodetype = atoi(iniStrPtr);
	}

	glocal(cfig);
	getdnserv(cfig);

	sprintf(cfig->servername_fqn, "%s.%s", cfig->servername, cfig->domainname);

	if (dhcpService)
	{
		cfig->socket[0] = socket(AF_INET,
		                         SOCK_DGRAM,
		                         IPPROTO_UDP);

		if (cfig->socket[0] == INVALID_SOCKET)
			return ;

		cfig->sockAddr[0].sin_family = AF_INET;
		cfig->sockAddr[0].sin_addr.s_addr = INADDR_ANY;
		cfig->sockAddr[0].sin_port = htons(IPPORT_DHCPS);

		int bOptVal = true;
		int bOptLen = sizeof(bOptVal);
		//setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *) &n, sizeof(n);
		setsockopt(cfig->socket[0], SOL_SOCKET, SO_REUSEADDR, (char*)&bOptVal, bOptLen);
		int bOptVal1 = true;
		int bOptLen1 = sizeof(bOptVal1);
		setsockopt(cfig->socket[0], SOL_SOCKET, SO_BROADCAST, (char*)&bOptVal1, bOptLen1);

		int nRet = bind(cfig->socket[0],
		                (LPSOCKADDR) & cfig->sockAddr[0],
		                sizeof(struct sockaddr_in)
		               );

		if (nRet == SOCKET_ERROR)
		{
			close(cfig->socket[0]);
			if (verbatim)
				printf("Error: Port 67 already in use\n");
			else
				syslog(LOG_MAKEPRI(LOG_LOCAL1, LOG_CRIT), "Port 67 already in use");
			scanf(tempbuff);
			exit(1);
		}

	}

	if (dnsService)
	{
		cfig->socket[1] = socket(AF_INET,
		                         SOCK_DGRAM,
		                         IPPROTO_UDP);

		if (cfig->socket[1] == INVALID_SOCKET)
			return ;

		cfig->sockAddr[1].sin_family = AF_INET;
		cfig->sockAddr[1].sin_addr.s_addr = cfig->serverip;
		cfig->sockAddr[1].sin_port = htons(IPPORT_DNS);

		int nRet = bind(cfig->socket[1],
		                (LPSOCKADDR) & cfig->sockAddr[1],
		                sizeof(struct sockaddr_in)
		               );
		if (nRet == SOCKET_ERROR)
		{
			if (cfig->socket[0])
				close(cfig->socket[0]);
			close(cfig->socket[1]);
			if (verbatim)
				printf("Error: Port 53 already in use\n");
			else
				syslog(LOG_MAKEPRI(LOG_LOCAL1, LOG_CRIT), "Port 53 already in use");
			exit(1);
		}

		cfig->socket[2] = socket(AF_INET,
		                         SOCK_DGRAM,
		                         IPPROTO_UDP);

		if (cfig->socket[2] == INVALID_SOCKET)
		{
			if (cfig->socket[0])
				close(cfig->socket[0]);
			if (cfig->socket[1])
				close(cfig->socket[1]);
			exit(1);
		}
	}

	if (getSection("STATIC-HOSTS", iniStr, sizeof(iniStr), iniFile))
	{
		char *iniStrPtr = cleanstr(iniStr, false);
		while (strlen(iniStrPtr))
		{
			char name[256];
			strcpy(name, iniStrPtr);
			char *value = strchr(name, 61);
			if (value != NULL)
			{
				*value = 0;
				value++;
				DWORD inetAddr = my_inet_addr(value);
				int i = getIndex(cfig, inetAddr);
				if (i >= 0)
				{
					strcpy(cfig->clients[i].hostname, name);
					cfig->clients[i].reserved = 1;
					cfig->clients[i].expiry = ULONG_MAX;
				}
			}
			iniStrPtr = cleanstr(iniStrPtr, true);
		}
	}
}

void glocal(data2 *cfig)
{
	gethostname(cfig->servername, sizeof(cfig->servername));
	if (!strlen(cfig->domainname))
		getdomainname(cfig->domainname, sizeof(cfig->domainname));
	if (!strncmp(cfig->domainname, "(none)", 6) || !strlen(cfig->domainname))
		strcpy(cfig->domainname, "workgroup");
	char buff[512];
	FILE *f = popen("/sbin/ifconfig", "r");
	bool eth = false;
	char *valueStart = NULL;
	char *valueEnd = NULL;
	char value[256];
	while (f && fgets(buff, sizeof(buff), f))
	{
		if (strstr(buff, "eth") == buff)
			eth = true;
		else if (strstr(buff, "  ") != buff)
			eth = false;

		if (eth && (valueStart = strstr(buff, "inet addr")))
		{
			valueStart += 10;
			valueEnd = strstr(valueStart, " ");
			if (!valueEnd)
				valueEnd = strstr(valueStart, "\n");
			if (valueEnd)
			{
				memcpy(value, valueStart, valueEnd - valueStart + 1);
				value[valueEnd - valueStart] = 0;
				DWORD addr = my_inet_addr(value);
				if (getNetwork(addr) == cfig->network)
					addEntry(cfig, cfig->servers, addr);
			}
		}
		if (eth && (valueStart = strstr(buff, "Mask")))
		{
			valueStart += 5;
			valueEnd = strstr(valueStart, " ");
			if (!valueEnd)
				valueEnd = strstr(valueStart, "\n");
			if (valueEnd)
			{
				memcpy(value, valueStart, valueEnd - valueStart + 1);
				value[valueEnd - valueStart] = 0;
				cfig->subnetmask = my_inet_addr(value);
			}
		}
	}
	if (f)
		pclose(f);

	if (!cfig->routers[0])
	{
		f = popen("/sbin/route", "r");
		while (f && fgets(buff, sizeof(buff), f))
		{
			if (strstr(buff, "default") == buff || strstr(buff, "0.0.0.0") == buff)
			{
				valueStart = buff + 16;
				valueEnd = valueStart + 16;
				memcpy(value, valueStart, valueEnd - valueStart + 1);
				value[valueEnd - valueStart] = 0;
				int addr = my_inet_addr(value);
				if (!addr)
				{
					char *dot = strchr(value, 46);
					if (dot)
						*dot = 0;
					hostent *ent = gethostbyname(value);
					if (ent)
						addr = *(DWORD*)ent->h_addr_list[0];
				}
				if (getNetwork(addr) == cfig->network)
					addEntry(cfig, cfig->routers, addr);
			}
		}
		if (f)
			pclose(f);
	}

	for (int i = 0; i < 5; i++)
		if (cfig->servers[i] == cfig->serverip)
			break;
		else if (i == 4)
		{
			if (verbatim)
				printf("Warning:Interface %s not found, ignored\n", IP2String(cfig->serverip));
			else
			{
				char tempbuff1[256];
				sprintf(tempbuff1, "Interface %s not found, ignored", IP2String(cfig->serverip));
				syslog(LOG_MAKEPRI(LOG_LOCAL1, LOG_NOTICE), tempbuff1);
			}
			cfig->serverip = INADDR_ANY;
		}
}

void getdnserv(data2 *cfig)
{
	char iniStr[4096];

	if (getSection("DNS-SERVERS", iniStr, sizeof(iniStr), iniFile))
	{
		char *iniStrPtr = cleanstr(iniStr, false);
		for (int i = 0; i < 5 && strlen(iniStrPtr); i++, iniStrPtr = cleanstr(iniStrPtr, true))
		{
			DWORD addr = my_inet_addr(iniStrPtr);
			if (addr)
				addEntry(cfig, cfig->dns, addr);
		}
	}

	if (!cfig->dns[0])
	{
		FILE *f = fopen("/etc/resolv.conf", "r");
		char buff[256];
		char *pin;
		DWORD addr;
		while (f && fgets(buff, 128, f))
		{
			if (pin = strstr(buff, "nameserver"))
			{
				pin += 11;
				DWORD addr = my_inet_addr(pin);
				for (int i = 0;i < 5;i++)
					if (cfig->servers[i] == addr)
						break;
					else if (i == 4)
						addEntry(cfig, cfig->dns, addr);
			}
		}
		if (f)
			fclose(f);
	}
}

char* cleanstr(char* buff, BOOL next)
{
	if (strlen(buff) && next)
		buff += strlen(buff) + 1;
	while (strlen(buff))
		if (*buff >= 48 && *buff <= 57 || *buff >= 65 && *buff <= 90 || *buff >= 97 && *buff <= 122)
			break;
		else
			buff += strlen(buff) + 1;
	return buff;
}

char* IP2String(DWORD ip)
{
	in_addr inaddr;
	inaddr.s_addr = ip;
	sprintf(tempbuff, "%s", inet_ntoa(inaddr));
	return tempbuff;
}

BOOL getSection(char *sectionName, char *buffer, int sizeofbuffer, char *fileName)
{
	char section[128];
	sprintf(section, "[%s]", sectionName);
	//strupr(section);
	FILE *f = fopen(fileName, "r");
	char buff[256];
	char *temp = buffer;
	*temp = 0;
	BOOL found = false;
	while (f && fgets(buff, 255, f))
	{
		//strupr(buff);
		if (strstr(buff, section) == buff)
		{
			found = true;
			while (fgets(buff, 255, f))
			{
				if (strstr(buff, "[") == buff)
					break;
				sprintf(temp, "%s", buff);
				temp += strlen(buff);
				while (*temp <= 32)
					temp--;
				temp++;
				*temp = 0;
				temp++;
			}
			*temp = 0;
		}
	}
	if (f)
		fclose(f);
	return found;
}

int getIndex(data2 *cfig, DWORD ip)
{
	DWORD ipTrans = htonl(ip);
	if (ipTrans >= cfig->rangeStart && ipTrans <= cfig->rangeEnd)
		return (ipTrans - cfig->rangeStart);
	else
		return -1;
}

DWORD getNetwork(DWORD ip)
{
	union {
		DWORD network;
		BYTE x[4];
	} data;

	data.network = ip;
	data.x[3] = 0;
	if (data.x[0] < 192)
		data.x[2] = 0;
	if (data.x[0] < 128)
		data.x[1] = 0;

	return data.network;
}


void addEntry(data2 *cfig, DWORD *array, DWORD entry)
{
	for (int i = 0; i < 5; i++)
		if (array[i] == entry)
			return ;
		else if (!array[i])
		{
			array[i] = entry;
			int j = getIndex(cfig, entry);
			if (j >= 0)
				cfig->clients[j].reserved = true;
			return ;
		}
}

DWORD my_inet_addr(char* str)
{
	if (str == NULL)
		return 0;
	DWORD x = inet_addr(str);
	if (x == INADDR_NONE)
		return INADDR_ANY;
	else
		return x;
}
