/***************************************************************************
 *   Copyright (C) 2005 by Achal Dhir                                      *
 *   achaldhir@gmail.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//This file defines all structures and constants
//for both DHCP and DNS Servers

#define WORD unsigned short
#define BYTE unsigned char
#define DWORD unsigned long
#define SOCKET_ERROR -1
#define INVALID_SOCKET -1
#define SOCKADDR_IN sockaddr_in
#define SOCKADDR sockaddr
#define SOCKET int
#define BOOL bool
#define LPSOCKADDR sockaddr*
#define closesocket close

#define RCODE_NOERROR 		0
#define RCODE_FORMATERROR	1
#define RCODE_SERVERFAIL	2
#define RCODE_NAMEERROR		3
#define RCODE_NOTIMPL 		4
#define RCODE_REFUSED 		5
#define RCODE_YXDOMAIN 		6
#define RCODE_YXRRSET 		7
#define RCODE_NXRRSET 		8
#define RCODE_NOTAUTH 		9
#define RCODE_NOTZONE 		10

#define OPCODE_STANDARD_QUERY	0
#define OPCODE_INVERSE_QUERY	1
#define OPCODE_SRVR_STAT_REQ	2
#define OPCODE_DYNAMIC_UPDATE	5

#define DNS_TYPE_A		1
#define DNS_TYPE_NS		2
#define DNS_TYPE_MD		3
#define DNS_TYPE_MF		4
#define DNS_TYPE_CNAME	5
#define DNS_TYPE_SOA	6
#define DNS_TYPE_MB		7
#define DNS_TYPE_MG		8
#define DNS_TYPE_MR		9
#define DNS_TYPE_NULL	10
#define DNS_TYPE_WKS	11
#define DNS_TYPE_PTR	12
#define DNS_TYPE_HINFO	13
#define DNS_TYPE_MINFO	14
#define DNS_TYPE_MX		15
#define DNS_TYPE_TXT	16
#define DNS_TYPE_AXFR	252
#define DNS_TYPE_MAILB	253
#define DNS_TYPE_MAILA	254
#define DNS_TYPE_ANY	255

#define DNS_CLASS_IN	1
#define DNS_CLASS_CS	2
#define DNS_CLASS_CH	3
#define DNS_CLASS_HS	4
#define DNS_CLASS_NONE	254
#define DNS_CLASS_ANY	255

#define IPPORT_DNS 53

struct dns_header {
	WORD xid;
	union
	{
		WORD filler;

		struct {
			BYTE rd:1;
			BYTE tc:1;
			BYTE aa:1;
			BYTE opcode:4;
			BYTE qr:1;
			BYTE rcode:4;
			BYTE z:3;
			BYTE ra:1;
		};
	};
	union {
		WORD zcount;
		WORD qdcount;
	};
	union {
		WORD prcount;
		WORD ancount;
	};
	union {
		WORD ucount;
		WORD nscount;
	};
	union {
		WORD adcount;
		WORD arcount;
	};
};

struct dns_packet {
	struct dns_header header;
	char data[512-sizeof(struct dns_header)];
};

struct data5 {
	SOCKADDR_IN sockAddr;
	union {
		dns_packet dnsp;
		char raw[sizeof(dns_packet)];
	};
	char raw_query[256];
	char query[256];
	WORD qtype;
	WORD qclass;
	union
	{
		char response[256];
		DWORD responselong;
	};
	WORD resp_len;
	int BytesRecd;
	int BytesReady;
	int BytesSent;
};

struct data6
{
	WORD xid;
	SOCKADDR_IN source;
	char hostname[256];
	DWORD ipaddr;
	WORD qtype;
};

struct data7
{
	char hostname[256];
	dns_packet dnsp;
	WORD qtype;
	time_t expiry;
	int bytes;
};

struct dns_rr {
	char *name;
	WORD type,_class;
	DWORD ttl;
	WORD rdlength;
	char *rdata;
	union {
		struct { long address;           } a;
		struct { char *cname;            } cname;
		struct { char *cpu,*os;          } hinfo;
		struct { char *madname;          } mb;
		struct { char *madname;          } md;
		struct { char *madname;          } mf;
		struct { char *mgmname;          } mg;
		struct { char *rmailbx,*emailbx; } minfo;
		struct { char *newname;          } mr;
		struct { int preference; char *exchange; } mx;
		struct { char *nsdname;          } ns;
		struct { char *data;             } null;
		struct { char *ptrdname;         } ptr;
		struct { char *mname,*rname; unsigned serial,refresh,retry,expire,minimum; } soa;
		struct { char **txt_data;        } txt;
		struct { int address; BYTE protocol; int bitmapsize; char *bitmap; } wks;
	} data;
};

#define DHCP_REQUEST  1
#define DHCP_REPLY    2

#define DHCP_MESS_NONE       0
#define DHCP_MESS_DISCOVER   1
#define DHCP_MESS_OFFER      2
#define DHCP_MESS_REQUEST	 3
#define DHCP_MESS_DECLINE	 4
#define DHCP_MESS_ACK		 5
#define DHCP_MESS_INFORM	 8


// DHCP OPTIONS

#define DHCP_OPTION_PAD						0
#define DHCP_OPTION_NETMASK          		1
#define DHCP_OPTION_TIMEOFFSET       		2
#define DHCP_OPTION_ROUTER           		3
#define DHCP_OPTION_TIMESERVER       		4
#define DHCP_OPTION_NAMESERVER       		5
#define DHCP_OPTION_DNS              		6
#define DHCP_OPTION_LOGSERVER        		7
#define DHCP_OPTION_COOKIESERVER     		8
#define DHCP_OPTION_LPRSERVER        		9
#define DHCP_OPTION_IMPRESSSERVER    		10
#define DHCP_OPTION_RESLOCSERVER     		11
#define DHCP_OPTION_HOSTNAME         		12
#define DHCP_OPTION_BOOTFILESIZE     		13
#define DHCP_OPTION_MERITDUMP        		14
#define DHCP_OPTION_DOMAINNAME       		15
#define DHCP_OPTION_SWAPSERVER       		16
#define DHCP_OPTION_ROOTPATH         		17
#define DHCP_OPTION_EXTSPATH         		18
#define DHCP_OPTION_IPFORWARD        		19
#define DHCP_OPTION_NONLOCALSR       		20
#define DHCP_OPTION_POLICYFILTER     		21
#define DHCP_OPTION_MAXREASSEMBLE    		22
#define DHCP_OPTION_IPTTL            		23
#define DHCP_OPTION_PATHMTUAGING     		24
#define DHCP_OPTION_PATHMTUPLATEAU   		25
#define DHCP_OPTION_INTERFACEMTU     		26
#define DHCP_OPTION_SUBNETSLOCAL     		27
#define DHCP_OPTION_BCASTADDRESS     		28
#define DHCP_OPTION_MASKDISCOVERY    		29
#define DHCP_OPTION_MASKSUPPLIER     		30
#define DHCP_OPTION_ROUTERDISCOVERY  		31
#define DHCP_OPTION_ROUTERSOLIC      		32
#define DHCP_OPTION_STATICROUTE      		33
#define DHCP_OPTION_TRAILERENCAPS    		34
#define DHCP_OPTION_ARPTIMEOUT       		35
#define DHCP_OPTION_ETHERNETENCAPS   		36
#define DHCP_OPTION_TCPTTL           		37
#define DHCP_OPTION_TCPKEEPALIVEINT  		38
#define DHCP_OPTION_TCPKEEPALIVEGRBG 		39
#define DHCP_OPTION_NISDOMAIN        		40
#define DHCP_OPTION_NISSERVERS       		41
#define DHCP_OPTION_NTPSERVERS       		42
#define DHCP_OPTION_VENDORSPECIFIC   		43
#define DHCP_OPTION_NETBIOSNAMESERV  		44
#define DHCP_OPTION_NETBIOSDGDIST    		45
#define DHCP_OPTION_NETBIOSNODETYPE  		46
#define DHCP_OPTION_NETBIOSSCOPE     		47
#define DHCP_OPTION_X11FONTS         		48
#define DHCP_OPTION_X11DISPLAYMNGR   		49
#define DHCP_OPTION_REQUESTEDIPADDR  		50
#define DHCP_OPTION_IPADDRLEASE      		51
#define DHCP_OPTION_OVERLOAD         		52
#define DHCP_OPTION_MESSAGETYPE      		53
#define DHCP_OPTION_SERVERID         		54
#define DHCP_OPTION_PARAMREQLIST     		55
#define DHCP_OPTION_MESSAGE          		56
#define DHCP_OPTION_MAXDHCPMSGSIZE   		57
#define DHCP_OPTION_RENEWALTIME      		58
#define DHCP_OPTION_REBINDINGTIME    		59
#define DHCP_OPTION_VENDORCLASSID    		60
#define DHCP_OPTION_CLIENTID         		61
#define DHCP_OPTION_NISPLUSDOMAIN    		64
#define DHCP_OPTION_NISPLUSSERVERS   		65
#define DHCP_OPTION_TFTPSERVER       		66
#define DHCP_OPTION_BOOTFILE         		67
#define DHCP_OPTION_MOBILEIPHOME     		68
#define DHCP_OPTION_SMTPSERVER       		69
#define DHCP_OPTION_POP3SERVER       		70
#define DHCP_OPTION_NNTPSERVER       		71
#define DHCP_OPTION_WWWSERVER        		72
#define DHCP_OPTION_FINGERSERVER     		73
#define DHCP_OPTION_IRCSERVER        		74
#define DHCP_OPTION_STSERVER         		75
#define DHCP_OPTION_STDASERVER       		76
#define DHCP_OPTION_USERCLASS        		77
#define DHCP_OPTION_SLPDIRAGENT      		78
#define DHCP_OPTION_SLPDIRSCOPE      		79
#define DHCP_OPTION_CLIENTFQDN       		81
#define DHCP_OPTION_NDSSERVERS       		85
#define DHCP_OPTION_NDSTREENAME      		86
#define DHCP_OPTION_NDSCONTEXT		 		87
#define DHCP_OPTION_AUTHENTICATION			90
#define DHCP_OPTION_CLIENTSYSTEM			93
#define DHCP_OPTION_CLIENTNDI				94
#define DHCP_OPTION_LDAP					95
#define DHCP_OPTION_UUID_GUID				97
#define DHCP_OPTION_USER_AUTH				98
#define DHCP_OPTION_NETINFOADDRESS			112
#define DHCP_OPTION_NETINFOTAG				113
#define DHCP_OPTION_URL						114
#define DHCP_OPTION_AUTO_CONFIG				116
#define DHCP_OPTION_NAMESERVICESEARCH		117
#define DHCP_OPTION_SUBNETSELECTION			118
#define DHCP_OPTION_DOMAINSEARCH			119
#define DHCP_OPTION_SIPSERVERSDHCP			120
#define DHCP_OPTION_CLASSLESSSTATICROUTE	121
#define DHCP_OPTION_CCC						122
#define DHCP_OPTION_GEOCONF					123
#define DHCP_OPTION_V_IVENDORCLASS			124
#define DHCP_OPTION_V_IVENDOR_SPECIFIC		125
#define DHCP_OPTION_TFPTSERVERIPADDRESS		128
#define DHCP_OPTION_CALLSERVERIPADDRESS		129
#define DHCP_OPTION_DISCRIMINATIONSTRING	130
#define DHCP_OPTION_REMOTESTATISTICSSERVER	131
#define DHCP_OPTION_802_1PVLANID			132
#define DHCP_OPTION_802_1QL2PRIORITY		133
#define DHCP_OPTION_DIFFSERVCODEPOINT		134
#define DHCP_OPTION_HTTPPROXYFORPHONE_SPEC	135
#define DHCP_OPTION_END						255

//#define DHCP_VENDORDATA_SIZE		 272
//#define DHCP_VENDORDATA_SIZE		 64
#define DHCP_VENDORDATA_SIZE		 784
#define DHCP_MIN_SIZE				 44
#define DHCP_MAX_CLIENTS			 254

struct dhcp_packet
{
	BYTE   bp_op;
	BYTE   bp_htype;
	BYTE   bp_hlen;
	BYTE   bp_hops;
	DWORD  bp_xid;
	WORD   bp_secs;
	WORD   bp_spare;
	DWORD  bp_ciaddr;
	DWORD  bp_yiaddr;
	DWORD  bp_siaddr;
	DWORD  bp_giaddr;
	BYTE   bp_chaddr[16];
	char   bp_sname[64];
	BYTE   bp_file[128];
	DWORD  bp_magic_num;
	BYTE   bp_vend[DHCP_VENDORDATA_SIZE];
};

#define IPPORT_DHCPS   67
#define IPPORT_DHCPC   68
#define VM_STANFORD  0x5354414EUL
#define VM_RFC1048   0x63825363UL
#define DNS_FORWARDS_MAX	32
#define DNS_CACHE_MAX		256


struct data1
{
	BYTE req_type;
	BYTE resp_type;
	union {
		dhcp_packet dhcpp;
		char raw[sizeof(dhcp_packet)];
	};
	char hostname[256];
	DWORD server;
	DWORD reqIP;
	int BytesRecd;
	int BytesSent;
	SOCKADDR_IN sockAddr;
};

struct data8
{
	char hostname[64];
	time_t expiry;
	BYTE chaddr[10];
	BYTE reserved;
};

struct data2
{
	//WSADATA wsaData;
	SOCKET socket[3];
	SOCKADDR_IN sockAddr[3];
	DWORD rangeStart;
	DWORD rangeEnd;
	DWORD serverip;
	char domainname[128];
	char servername[128];
	char servername_fqn[256];
	DWORD servers[5];
	BOOL routerSpecified;
	DWORD subnetmask;
	DWORD dns[5];
	DWORD routers[5];
	DWORD lease;
	DWORD network;
	BYTE nodetype;
	data6 queue[DNS_FORWARDS_MAX];
	data7 cache[DNS_CACHE_MAX];
	data8 clients[DHCP_MAX_CLIENTS];
	WORD queries_awaited;
};

struct data3
{
	BYTE opt_code;
	BYTE size;
	union {
		BYTE value[128];
		char charvalue[128];
		DWORD longvalue;
	};
};

